(* file: part1.ml
 * author: Bob Muller
 *
 * CS1101 Computer Science I
 *
 * Solution to part1 of problem set 3.
 *)

(* discount : int -> float
 *
 * The call (discount age) returns a percentage discount depending
 * on the age. Kids (under age 13) get in free (i.e., a 100% discount),
 * people otherwise under the age of 25, but not 19 year olds, receive
 * a 20% discount, seniors >= 65 receive a 30% discount. Otherwise 
 * your discount is 0%.
 *)
let discount age =
  match age < 13 with
  | true  -> 1.0
  | false -> (match age < 25 && (not (age = 19)) with
	      | true  -> 0.2
	      | false -> (match age > 64 with
			  | true  -> 0.3
			  | false -> 0.0))


let test () : bool =
  ((discount 19) = 0.0) &&
  ((discount 18) = 0.2) &&
  ((discount 45) = 0.0) &&
  ((discount 90) = 0.3);;

Assert.run_test "discount test" test

let rec fact n =
  match n = 0 with
  | true  -> 1
  | false -> n * (fact (n - 1))

(* euler : int -> float
 *
 * The call (euler n) returns the nth approximation of Euler's number
 *
 * e = 2.718281 ...
 *)	     
let rec euler n =
  match n = 0 with
  | true  -> 1.0
  | false -> let previous = euler (n - 1)
	     in
	     previous +. (1.0 /. (float (fact n)))

let test () : bool =
  (int_of_float (1000.0 *. (euler 6))) = 2718;;

Assert.run_test "euler test" test


let isFactor m n = n mod m = 0


(* firstFactor : int -> int
 *
 * The call (firstFactor n) returns the smallest integer factor of
 * n that is greater than 1.
 *)
let firstFactor n =
  let rec repeat m =
    match isFactor m n with
    | true  -> m
    | false -> repeat (m + 1)
  in
  repeat 2

let test () : bool =
  ((firstFactor 323) = 17) && (firstFactor 2) = 2;;

Assert.run_test "firstFactor test" test

(* isPerfect : int -> bool
 *
 * The call (isPerfect n) returns true if n is a perfect number and
 * false otherwise. A number is perfect if it is equal to the sum of
 * it's proper factors. E.g., 6 is perfect because 6 = 3 + 2 + 1.
 *)
let isPerfect n =
  let rec repeat m sum =
    match m > n / 2 with
    | true  -> sum = n
    | false -> (match isFactor m n with
		| true  -> repeat (m + 1) (sum + m)
		| false -> repeat (m + 1) sum)
  in
  repeat 1 0

let test () : bool =
  (isPerfect 6) && (isPerfect 28) && (not (isPerfect 12));;

Assert.run_test "isPerfect test" test

